datacache <- new.env(hash=TRUE, parent=emptyenv())

IlluminaHumanMethylation450k <- function() { # {{{
  showQCData("IlluminaHumanMethylation450k", datacache)
} # }}}
IlluminaHumanMethylation450k_dbconn <- function() dbconn(datacache)
IlluminaHumanMethylation450k_dbfile <- function() dbfile(datacache)
IlluminaHumanMethylation450k_dbschema <- function(file="", show.indices=FALSE) dbschema(datacache, file=file, show.indices=show.indices)
IlluminaHumanMethylation450k_dbInfo <- function() dbInfo(datacache)

########### BEGIN TJT MANUAL BIMAP ADDITIONS: PROBEDESIGN TABLE ################

## {{{ Most probes map to build 37 in the most recent v1.1 450k manifest
IlluminaHumanMethylation450kCPG37 <- createSimpleBimap("probedesign","Probe_ID","Coordinate_37",datacache,"CPGCOORDINATE","IlluminaHumanMethylation450k.db")
IlluminaHumanMethylation450kCHR37 <- createSimpleBimap("probedesign","Probe_ID","Chromosome_37",datacache,"CHR37","IlluminaHumanMethylation450k.db")
IlluminaHumanMethylation450kCPG36 <- createSimpleBimap("probedesign","Probe_ID","Coordinate_36",datacache,"CPG36","IlluminaHumanMethylation450k.db")
IlluminaHumanMethylation450kCHR36 <- createSimpleBimap("probedesign","Probe_ID","Chromosome_36",datacache,"CHR36","IlluminaHumanMethylation450k.db")
IlluminaHumanMethylation450kBUILD <- createSimpleBimap("probedesign","Probe_ID","Genome_Build",datacache,"BUILD","IlluminaHumanMethylation450k.db")
## }}}

## Could add a function to switch this back and forth between 36 and 37...
IlluminaHumanMethylation450kCPGCOORDINATE <- IlluminaHumanMethylation450kCPG37

## Irizarry-style CPG island designation
IlluminaHumanMethylation450kISCPGISLAND <- createSimpleBimap("probedesign","Probe_ID","Is_HMM_Island",datacache,"ISCPGISLAND","IlluminaHumanMethylation450k.db")

## UCSC CpG island name 
IlluminaHumanMethylation450kCPGINAME <- createSimpleBimap("cpgiview","Probe_ID","ucscname",datacache,"CPGINAME","IlluminaHumanMethylation450k.db")

## UCSC CpG island relationship 
IlluminaHumanMethylation450kCPGIRELATION <- createSimpleBimap("cpgiview","Probe_ID","relationship",datacache,"CPGIRELATION","IlluminaHumanMethylation450k.db")

## shelf/shore/whatever
IlluminaHumanMethylation450kCPGILOCATION<- createSimpleBimap("cpgilocation","Probe_ID","location",datacache,"CPGILOCATION","IlluminaHumanMethylation450k.db")

## UCSC regulatory region name 
IlluminaHumanMethylation450kREGULATORYNAME <- createSimpleBimap("regview","Probe_ID","ucsc_name",datacache,"REGULATORYNAME","IlluminaHumanMethylation450k.db")

## UCSC regulatory region relationship
IlluminaHumanMethylation450kREGULATORYGROUP <- createSimpleBimap("regview","Probe_ID","ucsc_group",datacache,"REGULATORYGROUP","IlluminaHumanMethylation450k.db")

## UCSC regulatory region 'location'
IlluminaHumanMethylation450kREGULATORYLOCATION <- createSimpleBimap("reglocation","Probe_ID","location",datacache,"REGULATORYLOCATION","IlluminaHumanMethylation450k.db")

## handy for finding out whether we're dealing with a promoter, body, UTR...
IlluminaHumanMethylation450kPROBELOCATION<- createSimpleBimap("probelocation","Probe_ID","location",datacache,"PROBELOCATION","IlluminaHumanMethylation450k.db")

## generated using `sapply(assay_450k[,14], function(x) seq2id(x))`
IlluminaHumanMethylation450kNUID <- createSimpleBimap("probedesign","Probe_ID","NuID",datacache,"NUID","IlluminaHumanMethylation450k.db")

## used `sapply(assay_450k[,14], function(x) length(gregexpr('(GC|CG)',x)[[1]])`
IlluminaHumanMethylation450kCPGS <- createSimpleBimap("probedesign","Probe_ID","CpGs",datacache,"CPGS","IlluminaHumanMethylation450k.db")

# DNAse hypersensitive sites
IlluminaHumanMethylation450kDHS <- createSimpleBimap("probedesign","Probe_ID","DHS",datacache,"DHS","IlluminaHumanMethylation450k.db")

# Enhancers 
IlluminaHumanMethylation450kENHANCER <- createSimpleBimap("probedesign","Probe_ID","enhancer",datacache,"ENHANCER","IlluminaHumanMethylation450k.db")

# SNPs within probe
IlluminaHumanMethylation450kSNP <- createSimpleBimap("probedesign","Probe_ID","snp",datacache,"SNP","IlluminaHumanMethylation450k.db")

# SNPs within 10bp of CpG 
IlluminaHumanMethylation450kSNP10 <- createSimpleBimap("probedesign","Probe_ID","snp10",datacache,"SNP10","IlluminaHumanMethylation450k.db")

# Strand (Illumina style)
IlluminaHumanMethylation450kSTRAND <- createSimpleBimap("probedesign","Probe_ID","strand",datacache,"STRAND","IlluminaHumanMethylation450k.db")

## for design II probes (aka 'Both'), Cy3 is methylated and Cy5 is unmethylated
IlluminaHumanMethylation450kDESIGN <- createSimpleBimap("probedesign","Probe_ID","Infinium_Design_Type",datacache,"DESIGN","IlluminaHumanMethylation450k.db")
IlluminaHumanMethylation450kRANDOM <- createSimpleBimap("probedesign","Probe_ID","Random_Loci",datacache,"RANDOM","IlluminaHumanMethylation450k.db")
IlluminaHumanMethylation450kMETHYL27 <- createSimpleBimap("probedesign","Probe_ID","Methyl27_Loci",datacache,"METHYL27","IlluminaHumanMethylation450k.db")
IlluminaHumanMethylation450kCOLORCHANNEL <- createSimpleBimap("probedesign","Probe_ID","Color_Channel",datacache,"COLORCHANNEL","IlluminaHumanMethylation450k.db")

######## SEPARATE QUERIES FOR DESIGN I, DESIGN II, AND CONTROL PROBES ##########

IlluminaHumanMethylation450k_getProbes <- function(design=NULL,color=NULL){ #{{{

  designs <- list(I='I',II='II')
  channels <- list(R='Red',G='Grn')
  if(is.null(design)) { 
    return(lapply(designs, function(d) {
      IlluminaHumanMethylation450k_getProbes(design=d, color=color)
    }))
  } else if(is.null(color) && design=='I') {
    return(lapply(channels, function(ch) {
      IlluminaHumanMethylation450k_getProbes(design=design, color=ch)
    }))
  } else {
    if(design == 'II') {
      designSql <- 'SELECT Probe_ID, M, U FROM probedesign2'
    } else if(design == 'I') {
      designSql <- 'SELECT Probe_ID, M, U FROM probedesign1'
      designSql <- paste(designSql, " WHERE Color_Channel='",color,"'", sep='')
    } else {
      stop('Unsupported design!')
    }
    dbGetQuery(IlluminaHumanMethylation450k_dbconn(), designSql)
  }

} # }}}

IlluminaHumanMethylation450k_getControls <- function(type=NULL) { # {{{

  controlSql <- 'SELECT * FROM controlprobes'
  if(!is.null(type)) {
    controlSql <- paste(controlSql," WHERE upper(Type) LIKE ")
    controlSql <- paste(controlSql, paste("'%", toupper(type), "%'", sep=''))
  }
  dbGetQuery(IlluminaHumanMethylation450k_dbconn(), controlSql)

} # }}}

IlluminaHumanMethylation450k_get27k <- function(design=NULL,color=NULL){ #{{{

  designs <- list(I='I',II='II')
  channels <- list(R='Red',G='Grn')
  if(is.null(design)) { 
    return(lapply(designs, function(d) {
      IlluminaHumanMethylation450k_get27k(design=d, color=color)
    }))
  } else if(is.null(color) && design=='I') {
    return(lapply(channels, function(ch) {
      IlluminaHumanMethylation450k_get27k(design=design, color=ch)
    }))
  } else {
    if(design == 'II') {
      designSql <- 'SELECT Probe_ID FROM probedesign2 WHERE Methyl27=1'
    } else if(design == 'I') {
      designSql <- 'SELECT Probe_ID FROM probedesign1 WHERE Methyl27=1'
      designSql <- paste(designSql, " AND Color_Channel='",color,"'", sep='')
    } else {
      stop('Unsupported design!')
    }
    dbGetQuery(IlluminaHumanMethylation450k_dbconn(), designSql)
  }

} # }}}

IlluminaHumanMethylation450k_getProbeOrdering <- function() { # {{{
  orderSql <- 'SELECT Probe_ID, 
                      Infinium_Design_Type AS DESIGN,
                      Color_Channel AS COLOR_CHANNEL
                 FROM probedesign 
             ORDER BY Probe_ID ASC'
  dbGetQuery(IlluminaHumanMethylation450k_dbconn(), orderSql)
} # }}}

################# SPARSE TABLES FOR FANTOM, DMR, DHS, ENHANCERS ################

IlluminaHumanMethylation450kFANTOM <- createSimpleBimap("fantom","Probe_ID","FANTOM",datacache,"FANTOM","IlluminaHumanMethylation450k.db")
IlluminaHumanMethylation450kDMR <- createSimpleBimap("dmr","Probe_ID","DMR",datacache,"DMR","IlluminaHumanMethylation450k.db")

################# SIMPLE IN-HOUSE REVISION TRACKING FOR EGC ####################

IlluminaHumanMethylation450kBLAME <- "$Author: ttriche $"
IlluminaHumanMethylation450kSVNID <- "$Id: zzz.R 1175 2011-01-24 23:22:52Z ttriche $"

################## END OF TJT ADDITIONS TO STOCK FUNCTIONS #####################

IlluminaHumanMethylation450kORGANISM <- "Homo sapiens"

.onLoad <- function(libname, pkgname) { # {{{ 
    require("methods", quietly=TRUE)
    ## Connect to the SQLite DB
    dbfile <- system.file("extdata", "IlluminaHumanMethylation450k.sqlite", package=pkgname, lib.loc=libname)
    assign("dbfile", dbfile, envir=datacache)
    dbconn <- dbFileConnect(dbfile)
    assign("dbconn", dbconn, envir=datacache)
    ## Create the AnnObj instances
    ann_objs <- createAnnObjs.SchemaChoice("HUMANCHIP_DB", "IlluminaHumanMethylation450k", "chip IlluminaHumanMethylation450k", dbconn, datacache)
    mergeToNamespaceAndExport(ann_objs, pkgname)
} # }}}

.onUnload <- function(libpath) { # {{{
    dbFileDisconnect(IlluminaHumanMethylation450k_dbconn())
} # }}}
